import { defineStore } from 'pinia'
import { useDialogueStore } from '@/store'
import lodash from 'lodash'

import { ref } from 'vue'
import { createGlobalState, useStorage } from '@vueuse/core'
import { uniStorage } from '@/utils/uniStorage.js'

export const useDialogueListStore = createGlobalState(() => {
  const testDatabase = async () => {
    // 初始化数据库
    let chatDatabase = {
      eventType: 'openDatabase',
      eventParams: {
        name: 'chat',
        path: '_doc/chat.db',
      },
    }
    let chatDBexecuteSql = {
      eventType: 'executeSql',
      eventParams: {
        name: 'chat',
        sql: `CREATE TABLE IF NOT EXISTS talk_records (
          id INTEGER PRIMARY KEY AUTOINCREMENT,
          msg_id TEXT NOT NULL,
          sequence INTEGER NOT NULL,
          talk_type INTEGER NOT NULL DEFAULT 1,
          msg_type INTEGER NOT NULL DEFAULT 1,
          user_id INTEGER NOT NULL DEFAULT 0,
          receiver_id INTEGER NOT NULL DEFAULT 0,
          is_revoke INTEGER NOT NULL DEFAULT 0,
          is_mark INTEGER NOT NULL DEFAULT 0,
          quote_id TEXT NOT NULL,
          extra TEXT NOT NULL,
          created_at TEXT NOT NULL,
          updated_at TEXT NOT NULL,
          biz_date TEXT
        )`,
      },
    }

    const content = {
      content: "我试试传送文件和图片是不是一个接口",
      name: "测试excel1.xlsx",
      path: "https://cdn-test.szjixun.cn/fonchain-chat/chat/file/multipart/20250307/727a2371-ffc4-46da-b953-a7d449ff82ff-测试excel1.xlsx",
      size: 9909,
      drive: 3
    };
    const extra = JSON.stringify(content);

    let chatDBexecuteSql2 = {
      eventType: 'executeSql',
      eventParams: {
        name: 'chat',
        sql: 'INSERT INTO talk_records (msg_id, sequence, talk_type, msg_type, user_id, receiver_id, is_revoke, is_mark, quote_id, extra, created_at, updated_at, biz_date) VALUES ("'+'77b715fb30f54f739a255a915ef72445'+'", 166, 2, 1, 1774, 888890, 0, 0, "'+''+'", "'+extra+'", "'+'2025-03-06T15:57:07.000Z'+'", "'+'2025-03-06T15:57:07.000Z'+'", "'+'20250306'+'")',
      },
    }
    let chatDBSelectSql = {
      eventType: 'selectSql',
      eventParams: {
        name: 'chat',
        sql: `SELECT * FROM talk_records ORDER BY sequence DESC LIMIT 20`,
      },
    }
    let chatDBIsOpenDatabase = {
      eventType: 'isOpenDatabase',
      eventParams: {
        name: 'chat',
        path: '_doc/chat.db',
      },
    }
    document.addEventListener('plusready', () => {
      let OAWebView = plus.webview.all()
      OAWebView.forEach((webview, index) => {
        if (webview.id === 'webviewId1') {
          webview.evalJS(
            `operateSQLite('${encodeURIComponent(
              JSON.stringify(chatDatabase),
            )}')`,
          )
          webview.evalJS(
            `operateSQLite('${encodeURIComponent(
              JSON.stringify(chatDBexecuteSql),
            )}')`,
          )
          webview.evalJS(
            `operateSQLite('${encodeURIComponent(
              JSON.stringify(chatDBexecuteSql2),
            )}')`,
          )
          webview.evalJS(
            `operateSQLite('${encodeURIComponent(
              JSON.stringify(chatDBSelectSql),
            )}')`,
          )
          webview.evalJS(
            `operateSQLite('${encodeURIComponent(
              JSON.stringify(chatDBIsOpenDatabase),
            )}')`,
          )
        }
      })
    })
  }
  // testDatabase()

  const dialogueList = useStorage('dialogueList', [], uniStorage)
  // const dialogueList = ref([])
  const zpagingRef = ref()
  const virtualList = ref([])

  const getDialogueList = (indexName) => {
    return dialogueList.value.find((item) => item.index_name === indexName)
  }

  const addDialogueRecord = (newRecords, type = 'add') => {
    console.log(newRecords)

    const dialogue = useDialogueStore()
    if (!dialogue || typeof dialogue !== 'object') return
    // 检查是否已存在相同 index_name 的对话
    const existingIndex = dialogueList.value.findIndex(
      (item) => item.index_name === dialogue.index_name,
    )
    if (existingIndex === -1) {
      // 如果不存在，创建新对话，只保存需要的属性
      const newDialogue = {
        index_name: dialogue.index_name,
        talk: {
          username: dialogue.talk.username,
          talk_type: dialogue.talk.talk_type,
          receiver_id: dialogue.talk.receiver_id,
        },
        online: dialogue.online,
        records: dialogue.records || [],
        unreadBubble: dialogue.unreadBubble,
        isOpenMultiSelect: dialogue.isOpenMultiSelect,
        isShowEditor: dialogue.isShowEditor,
        isShowSessionList: dialogue.isShowSessionList,
        isDismiss: dialogue.isDismiss,
        isQuit: dialogue.isQuit,
        unreadNum: dialogue.unreadNum,
        members: dialogue.members.map((member) => ({
          id: member.id,
          nickname: member.nickname,
          avatar: member.avatar,
          gender: member.gender,
          leader: member.leader,
          remark: member.remark,
          online: member.online,
          value: member.value,
          key: member.key,
          erp_user_id: member.erp_user_id,
          is_mute: member.is_mute,
          is_mine: member.is_mine,
        })),
        forwardType: dialogue.forwardType,
      }
      dialogueList.value.push(newDialogue)
    } else {
      // 如果对话存在，处理 records 数组
      newRecords.forEach((newRecord) => {
        const recordIndex = dialogueList.value[existingIndex].records.findIndex(
          (record) => record.msg_id === newRecord.msg_id,
        )

        if (recordIndex === -1) {
          // 如果记录不存在，添加到 records 数组
          if (type === 'add') {
            dialogueList.value[existingIndex].records.push(newRecord)
          } else {
            dialogueList.value[existingIndex].records.unshift(newRecord)
          }
        }
      })
    }
  }

  const updateDialogueRecord = (record) => {
    const dialogue = useDialogueStore()
    const item = getDialogueList(dialogue.index_name)
    const recordIndex = item.records.findIndex(
      (item) => item.msg_id === record.msg_id,
    )
    if (recordIndex !== -1) {
      item.records[recordIndex] = {
        ...item.records[recordIndex],
        ...record,
      }
    }
    const virtualIndex = virtualList.value.findIndex(
      (item) => item.msg_id === record.msg_id,
    )
    if (virtualIndex !== -1) {
      virtualList.value[virtualIndex] = {
        ...virtualList.value[virtualIndex],
        ...record,
      }
    }
  }

  const deleteDialogueRecord = (record) => {
    const dialogue = useDialogueStore()
    const item = getDialogueList(dialogue.index_name)
    const recordIndex = item.records.findIndex(
      (item) => item.msg_id === record.msg_id,
    )
    if (recordIndex !== -1) {
      item.records.splice(recordIndex, 1)
    }
  }

  const updateUploadProgress = (id, progress) => {
    const record = virtualList.value.find((item) => item.msg_id === id)
    if (record) {
      record.uploadCurrent = progress
    }
  }

  const updateZpagingRef = (params) => {
    zpagingRef.value = params
  }

  const zpagingComplete = (index_name, newRecords) => {
    zpagingRef.value?.complete(lodash.cloneDeep(newRecords).reverse())
  }

  const addChatRecord = (indexName, item) => {
    const dialogue = useDialogueStore()
    if (dialogue?.index_name === indexName) {
      if (item?.file_num) {
        const index = virtualList.value.findIndex(
          (v) => v?.file_num === item?.file_num,
        )
        if (index > -1) {
          // 保持响应性的同时替换整个对象
          virtualList.value.splice(index, 1, {
            ...virtualList.value[index], // 保留原有不需要修改的字段
            ...item, // 覆盖需要更新的字段
          })
        } else {
          zpagingRef.value?.addChatRecordData(item, false, false)
        }
      } else {
        zpagingRef.value?.addChatRecordData(item, false, false)
      }
    }
  }

  const batchDelDialogueRecord = (msgIds) => {
    const dialogue = useDialogueStore()
    const item = getDialogueList(dialogue.index_name)
    item.records = item.records.filter((item) => !msgIds.includes(item.msg_id))
  }

  //删除会话时，同时刪除storage中存儲的會話
  const delDialogueStorage = (indexName) => {
    if (dialogueList?.value?.length > 0) {
      dialogueList.value.forEach((item, index) => {
        if (item?.index_name === indexName) {
          dialogueList.value.splice(index, 1)
        }
      })
    }
  }

  //清空聊天记录时，同时清空本地保存的聊天记录
  const clearDialogueRecord = () => {
    const dialogue = useDialogueStore()
    const item = getDialogueList(dialogue.index_name)
    item.records = []
    virtualList.value = []
  }

  return {
    dialogueList,
    zpagingRef,
    getDialogueList,
    addDialogueRecord,
    updateDialogueRecord,
    deleteDialogueRecord,
    updateUploadProgress,
    updateZpagingRef,
    zpagingComplete,
    addChatRecord,
    virtualList,
    batchDelDialogueRecord,
    delDialogueStorage,
    clearDialogueRecord,
  }
})
